@@ -7,12 +7,13 @@ use AE;
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket qw( tcp_connect );
+use Protocol::WebSocket::Request;
 use Protocol::WebSocket::Handshake::Client;
 use AnyEvent::WebSocket::Connection;
 use PerlX::Maybe qw( maybe provided );
 
 # ABSTRACT: WebSocket client for AnyEvent
-our $VERSION = '0.35_01'; # TRIAL VERSION
+our $VERSION = '0.35_02'; # TRIAL VERSION
 
 
 has timeout => (
@@ -42,6 +43,26 @@ has subprotocol => (
 );
 
 
+has http_headers => (
+  is => 'ro',
+  coerce => sub {
+    ref $_[0] eq 'ARRAY' ? $_[0] : do {
+      my $h = shift;
+      [
+        map {
+          my($k,$v) = ($_, $h->{$_});
+          $v = [$v] unless ref $v;
+          map { $k => $_ } @$v;
+          # sorted to make testing easier.
+          # may be removed in the future
+          # so do not depend on it.
+        } sort keys %$h
+      ],
+    };
+  },
+);
+
+
 sub connect
 {
   my($self, $uri) = @_;
@@ -68,9 +89,11 @@ sub connect
       $done->croak("unable to connect");
       return;
     }
+    my $req = Protocol::WebSocket::Request->new( maybe headers => $self->http_headers );
     my $handshake = Protocol::WebSocket::Handshake::Client->new(
             url     => $uri->as_string,
       maybe version => $self->protocol_version,
+            req     => $req,
     );
     
     my %subprotocol;
@@ -156,7 +179,7 @@ AnyEvent::WebSocket::Client - WebSocket client for AnyEvent
 
 =head1 VERSION
 
-version 0.35_01
+version 0.35_02
 
 =head1 SYNOPSIS
 
@@ -246,6 +269,34 @@ WebSocket protocol versions.
 List of subprotocols to request from the server.  This class will throw an
 exception if none of the protocols are supported by the server.
 
+=head2 http_headers
+
+Extra headers to include in the initial request.  May be either specified
+as a hash reference, or an array reference.  For example:
+
+ AnyEvent::WebSocket::Client->new(
+   http_headers => {
+     'X-Foo' => 'bar',
+     'X-Baz' => [ 'abc', 'def' ],
+   },
+ );
+ 
+ AnyEvent::WebSocket::Client->new(
+   http_headers => [
+     'X-Foo' => 'bar',
+     'X-Baz' => 'abc',
+     'X-Baz' => 'def',
+   ],
+ );
+
+Will generate:
+
+ X-Foo: bar
+ X-Baz: abc
+ X-Baz: def
+
+Although, the order cannot be guaranteed when using the hash style.
+
 =head1 METHODS
 
 =head2 connect
